//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CalView.cc	--- implementation of CalView class methods
//
//	DESCRIPTION:
//		This is the implementation part of the CalView classes
//
//	CHANGE HISTORY:
//		UL, 1999-03-23: 1.0.0: - first version
//		UL, 1999-09-15: 1.1.0: - added AbsCoefPlot
//		UL, 1999-10-21: 1.2.0: - added HKParmIntPlot
//
//	AUTHOR:
//		U. Lammers, Wed Mar 24 16:24:14 MET 1999
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /usr/local/cvs/sasdev/sas/packages/cal/calview/CalView.cc,v 1.29 2002/07/26 15:07:37 ulammers Exp $
//

#include "CalServer.h"
#include "CalView.h"
#include "CalViewables.h"
#include "ExceptionErrorHandler.h"
#include "errstr.h"

CalViewServer &calViewServerFunc()
{
	static CalViewServer *calViewServerInstance = new CalViewServer;

	return *calViewServerInstance;
} // calViewServerFunc()


//
//	---( CalViewable )---------------------------------------------------
//
CalViewable::CalViewable(const CalViewServer *server)
	: CalChild<CalViewServer>(server)
{
} // CalViewable::CalViewable()


CalViewProductDescriptor CalViewable::view(const string &setName) const
{
	errstr<<"The viewable `"<<name()<<"' cannot be viewed"
		  <<Sas::warning("CalViewable::view");

	return CalViewProductDescriptor(setName);
} // CalViewable::view()


const string &CalViewable::name() const
{
	return _name;
} // CalViewable::name()


const string &CalViewable::whatIs() const
{
	return _whatIs;
} // CalViewable::whatIs()


bool CalViewable::isApplicable(Xmm::Instrument) const
{
	return true;
} // CalViewable::isApplicable()


unsigned CalViewable::subViewables() const
{
	return _subViewable.size();
} // CalViewable::subViewables()


const vector<CalViewable *> &CalViewable::subViewable() const
{
	return _subViewable;
} // CalViewable::subViewable()


CalViewable::~CalViewable()
{
} // CalViewable::~CalViewable()


//
//	---( CalViewServer )---------------------------------------------------
//
CalViewServer::CalViewServer()
{
	#define registerViewable(Type)								\
	{	CalViewable *p = new Type(this);						\
		_viewable.push_back(p); }

	registerViewable(AbsCoefPlot);
	registerViewable(BadPixPlot);
	registerViewable(CcdPlot);
	registerViewable(CrossPsfPlot);
	registerViewable(DefocusLengthPlot);
	registerViewable(DispersedPsfPlot);
	registerViewable(EffectiveAreaPlot);
	registerViewable(EncircledEnergyPlot);
	registerViewable(FilterTransPlot);
	registerViewable(GratingPropertiesPlot);
	registerViewable(HKParmIntPlot);
	registerViewable(PsfPlot);
	registerViewable(QuantumEfficiencyPlot);
	registerViewable(RedistributionPlot);
	registerViewable(VignettingPlot);

	#undef registerViewable
} // CalViewServer::CalViewServer()


CalViewState *CalViewServer::state()
{
	return &_state;
} // CalViewServer::state()


unsigned CalViewServer::viewables() const
{
	return _viewable.size();
} // CalViewServer::viewables()


vector<const CalViewable *> CalViewServer::viewable() const
{
	vector<const CalViewable *> vv;
	Xmm::Instrument instrument = calServer.instrument()->id();
	for (unsigned n=0; n<_viewable.size(); ++n)
		if (_viewable[n]->isApplicable(instrument))
			vv.push_back(_viewable[n]);

	return vv;
} // CalViewServer::viewable()


CalViewServer::~CalViewServer()
{
	for (unsigned n=0; n<_viewable.size(); delete _viewable[n++]);
} // CalViewServer::CalViewServer()
