/*-----------------------------------------------------------------------------
 *
 *						XMM Science Analysis System (SAS)
 *						(c) 1997 European Space Agency
 *
 *-----------------------------------------------------------------------------
 *
 *	CalviewBackend.h: --- the CAL from C ---
 *
 *	DESCRIPTION:
 *		This header file acts as the interface between the calview
 *		front- and back-end. It is in ANSI C-style because the
 *		front-end requires this.
 *
 *	CHANGE HISTORY:
 *		UL, 08/05/98: 1.0.0: - first version
 *
 *	AUTHOR:
 *		U. Lammer, Fri May  8 11:14:30 MET DST 1998
 *
 *	REFERENCES:
 *		[1]: Software Specification Document for the XMM Science Analysis
 *			 Subsystem, M. Dahlem et. al., 
 *		[2]: TBW note about calview
 *
 *	$Header: /usr/local/cvs/sasdev/sas/packages/cal/calview/CalviewBackend.h,v 1.4 2002/05/24 10:59:16 gvacanti Exp $
 */

#ifndef CALVIEW_BACKEND_H
#define CALVIEW_BACKEND_H

/*
 *	instruments, data modes, filters
 */
typedef enum {
	INSTRUMENT_NONE = -1,
	SC, XRT1, XRT2, XRT3, EMOS1, EMOS2, EPN, ERM, RGS1, RGS2, OM 
} XmmInstruments;

typedef enum {
	MODE_NONE = -1,

	/*	EMOS */
	IMAGING, TIMING, REDUCED_IMAGING, COMPRESSED_TIMING, AUXILIARY,
	DIAGNOSTIC, OFFSET_VARIANCE, PERIODIC_HOUSEKEEPING,

	/*	EPN not in above list */
	BURST, OFFSET_DATA, NOISE_DATA, DISCARDED_LINES_DATA,
	MAIN_PERIODIC_HOUSEKEEPING, ADD_PERIODIC_HOUSEKEEPING,

	/*	ERM not in avove list */
	ERM_COUNT_RATE, ERM_SPECTRA,

    /*	RGS not in above list */
	SPECTROSCOPY, HIGH_TIME_RESOLUTION, FULL_PERIODIC_HOUSEKEEPING,
	SUMMARY_PERIODIC_HOUSEKEEPING, TEMP_PERIODIC_HOUSEKEEPING,

	/* 	OM not in above list */
	FAST, ENGINEERING1, ENGINEERING2, ENGINEERING3, ENGINEERING4,
    ENGINEERING5, ENGINEERING6, PRIORITY_FIELD_ACQUISITION,
	PRIORITY_WINDOW_DATA, PRIORITY_FAST, TRACKING_HISTORY,
	REFERENCE_FRAME, NON_PERIODIC_HOUSEKEEPING,

	/*	S/C not in above list */
	ATTITUDE, DUMMY_ATTITUDE, PREDICTED_ORBIT, RECONSTRUCTED_ORBIT,
	TIME_CORRELATION, HK_P1, HK_P2, HK_P3, HK_P4, HK_P5, HK_P6,

	/*	general */
	SUMMARY_INFORMATION
} XmmDataModes;

typedef enum {
	FILTER_NONE = -1,
	OM_UVW2, OM_UVM2, OM_UVW1, OM_U, OM_B, OM_V,
	OM_WHITE, OM_MAGNI, OM_GRISM1, OM_GRISM2
} XmmFilters;


typedef enum {
	LOW_LEVEL = 0, MEDIUM_LEVEL, HIGH_LEVEL
} AccuracyLevels;

#if !defined(__cplusplus)
typedef enum {
	false = 0, true
} bool;
#endif


/*
 *	the CAL state structure
 */
typedef struct {
	XmmInstruments	instrument;
	unsigned		ccdChip;
	unsigned		ccdNode;
	XmmDataModes	dataMode;
	XmmFilters		filter;
	double			temperature;
	double			observationStartTime;
	AccuracyLevels	accuracyLevel;
	bool			randomizeSwitch;
} CalState;


/*
 *	CAL atoms
 */
typedef enum {
	ATOM_NONE = -1,
	AbsCoefs,
	ADUConv,
	AstroMet,
	Background,
	BadPix,
	BoreSight,
	CCDrmf,
	CTI,
	CalIndex,
	CalSourceData,
	ColorTrans,
	Contam,
	CrossPSF,
	DarkCurrent,
	DarkFrame,
	DiffuseGala,
	EvtSizeDist,
	EXAFS,
	FilterTransV,
	FilterTransX,
	HKParmInt,
	LeapSeconds,
	LightLeak,
	LinCoord,
	LineSpreadFunc,
	LargeScaleSens,
	MiscData,
	ModeParam,
	PatternLib,
	PixToPixSens,
	PhotToNat,
	PhotVsLam,
	PixToLam,
	PSF1DRB,
	PSFPileUp,
	QuantumEf,
	QuickMag,
	Redist,
	RedistPileUp,
	StrayLight,
	TimeCorr,
	VAreaEf,
	VPSF,
	VSLAreaEf,
	VSLDist,
	XAreaEf,
	XEncirEn,
	XPSF,
	XSLAreaEf,
	XSLDist,
	Zodiacal
} CalAtoms;


/*
 *	the view state structure
 */
typedef struct {
	double			E;
	double			theta, phi;
	char			*dumpFileName;	
} ViewState;


#if defined(__cplusplus)
extern "C" {
#endif

/*	open a CCF */
void			openCcf(const char *cifName);

/*	register a CCF constituent */
void			setCalibrationFile(const char *calibrationFileName);

/*	set/get the CAL state */
void			setCalState(const CalState *calState);
void			setCalStateFromScienceFile(const char *fileName);
const CalState	*getCalState(void);

/*	set/get the view state */
void			setViewState(const ViewState *viewState);
const ViewState	*getViewState(void);

/*	view/dump CAL atoms */
void		view(CalAtoms calAtom);
void		dump(CalAtoms calAtom);

#if defined(__cplusplus)
		   } /* extern "C" */
#endif

#endif


