!
!   epntime.f90  --- test fine time adjustments for EPN
!
program epntime

use cal
use types
use caloaldefs
use errorhandling

    integer                             :: mode, ccd, i
    real(double), dimension(0:0)        :: time
    integer(kind=int16), dimension(0:0) :: rawY
    integer(kind=int16)                 :: sourceY
    logical                             :: isClose
    real(double), dimension(0:40)       :: nomTime = (/ &
      -51.1599998474121094, -45.1100006103515625, -38.9599990844726634, &
      -51.1599998474121094, -45.1100006103515625, -38.9599990844726634, &
      -51.1599998474121094, -45.1100006103515625, -38.9599990844726634, &
      -51.1599998474121094, -45.1100006103515625, -38.9599990844726634, &
      -1.1405000305175781E+2, -1.0800000000000000E+2, -1.0184999847412109E+2, &
      -1.1405000305175781E+2, -1.0800000000000000E+2, -1.0184999847412109E+2, &
      -1.1405000305175781E+2, -1.0800000000000000E+2, -1.0184999847412109E+2, &
      -1.1405000305175781E+2, -1.0800000000000000E+2, -1.0184999847412109E+2, &
      -33.0000000000000000, -29.0300006866455078, -25.0699996948242188, &
      -33.0000000000000000, -29.0300006866455078, -25.0699996948242188, &
      -33.0000000000000000, -29.0300006866455078, -25.0699996948242188, &
      -33.0000000000000000, -29.0300006866455078, -25.0699996948242188, &
      -3.6700000762939453, -4.2699999809265137, -1.4759999513626099, &
      -2.3911199513822794, -4.1566799004795021/)

    call CAL_setState(EPN)

    !   test three main imaging modes
    i = 0

    mode = PRIME_FULL_WINDOW
    write(*, *) 'In mode: ', modeId(mode)
    call CAL_setState(ccdModeId=mode)
    do ccd=EPN_CCD_FIRST, EPN_CCD_LAST
        call CAL_setState(ccdChipId=ccd)
        time(0) = 0
        call CAL_correctTime(time)
        write(*, *) 'CCD ', ccd, ' time=', 1000.*time(0), ' ms'
        if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
        i = i + 1
    enddo

    do mode=PRIME_FULL_WINDOW_EXTENDED, PRIME_LARGE_WINDOW
        write(*, *) 'In mode: ', modeId(mode)
        call CAL_setState(ccdModeId=mode)
        do ccd=EPN_CCD_FIRST, EPN_CCD_LAST
            call CAL_setState(ccdChipId=ccd)
            time(0) = 0
            call CAL_correctTime(time)
            write(*, *) 'CCD ', ccd, ' time=', 1000.*time(0), ' ms'
            if (.not. isClose(1000.*time(0), nomTime(i))) &
                 call error("Something wrong with fine time")
            i = i + 1
        enddo
    enddo



    !   test small window mode - only CCD 4 can be active
    mode = PRIME_SMALL_WINDOW
    call CAL_setState(ccdModeId=mode, ccdChipId=EPN_CCD_4)
    time(0) = 0.
    call CAL_correctTime(time)
    write(*, *) 'In mode: ', modeId(mode)
    write(*, *) 'CCD ', EPN_CCD_4, ' time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    !   burst mode
    mode = FAST_BURST
    call CAL_setState(ccdModeId=mode)
    time(0) = 0.
    call CAL_correctTime(time)
    write(*, *) 'In mode: ', modeId(mode)
    write(*, *) 'CCD ', EPN_CCD_4, ' time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    !   timing and burst mode with and without PSF position
    rawY(0) = 129
    write(*, *) 'Timing mode without source position, rawY =', rawY(0)
    mode = FAST_TIMING
    call CAL_setState(ccdModeId = mode)
    write(*, *) 'In mode: ', modeId(mode)
    time(0) = 0.
    call CAL_correctTime(time, rawY)
    write(*, *) 'time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    sourceY = 100
    write(*, *) 'Timing/burst mode: rawY=', rawY(0), 'src pos=', sourceY
    do mode=FAST_TIMING, FAST_BURST
        call CAL_setState(ccdModeId = mode)
        write(*, *) 'In mode: ', modeId(mode)
        time(0) = 0.
        call CAL_correctTime(time, rawY, sourceY)
        write(*, *) 'time=', 1000.*time(0), ' ms'
        if (.not. isClose(1000.*time(0), nomTime(i))) &
                 call error("Something wrong with fine time")
        i = i + 1
    enddo

    call message(AppMsg, Verbose,"all tests passed")
end program


logical function isClose(value, trueval)
use types

    real(double), intent(in)    :: value, trueval
    real(double)                :: absval, abstrueval

    isClose = .true.
    if (value * trueval .lt. 0.) isClose = .false.
    absval = abs(value)
    abstrueval = abs(trueval)
    if (absval .lt. .99999 * abstrueval .or. &
        absval .gt. 1.0001 * abstrueval) isClose = .false.
end function

