//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	BadPixelDataServer.h --- classes related to bad pixels
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 2001-02-19: 1.0.0: - first version from CalAtomsCommon.h
//		UL, 2001-03-22: 1.1.0: - moved BadPixel/BadPixelMap classes to
//								 caloalutils
//		UL, 2001-12-05: 1.2.0: - added BadPixelDataServer::badPixelTableCode()
//
//	AUTHOR:
//		U. Lammers, Mon Oct  6 15:32:57 WET DST 1997
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /usr/local/cvs/sasdev/sas/packages/cal/src/BadPixelDataServer.h,v 1.9 2002/05/24 10:59:16 gvacanti Exp $
//

#ifndef BAD_PIXEL_DATA_SERVER_H
#define BAD_PIXEL_DATA_SERVER_H

#include "Ccf.h"
#include "CalAtomsBase.h"
#include "BadPixelMap.h"

#include <vector>

class BadPixelDataServer : public CalAtom,
	public virtual CalChild<CalAtomCollection<BadPixelDataServer> > {
public:
	BadPixelDataServer(const string &bptCode,
					   CalAtomCollection<BadPixelDataServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate bad pixel data server object
		//


	static unsigned id() { return Ccf::BadPix; }
		//
		//	PURPOSE:
		//		Return ID of CalAtom
		//

	static const string INVALID_BAD_PIXEL_TABLE_CODE;
	const string &badPixelTableCode() const { return _bptCode; }
		//
		//	PURPOSE:
		//		Return Bad-Pixel-Table code read from attribute
		//		*BADPIX*:BADPIX%BPT_CODE; returns
		//		BadPixelDataServer::INVALID_BPT_CODE if not present
		//		


	BadPixelMap map() const;
		//
		//	PURPOSE:
		//		Obtain the list of bad pixels applicable for current
		//		instrument mode.
		//


	~BadPixelDataServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	friend class CalAtomCollection<BadPixelDataServer>;

	typedef struct {
		PixelLocation			loc;
		BadPixel				bp;
		std::vector<short>		modena;
	} LocBadPixModeType;
	std::vector<LocBadPixModeType>	_badPixels;
	std::string						_bptCode;
};

#endif

